package stamp.peripheral.wireless.linkmatik;

import stamp.core.*;
import stamp.util.text.*;

public class Channel extends Serial {

  public static final int  MAX_DATA_SIZE   = 100;
  public static final int  CONTROL_OFFSET  = 4;
  public static final int  CONTROL_ATERSET = 1;
  public static final int  MAX_BUFFER_SIZE = CONTROL_OFFSET + MAX_DATA_SIZE + CONTROL_ATERSET;

  char[]                   frame           = new char[MAX_BUFFER_SIZE];

  public static final char SOF             = (char) 0xBF;
  // Start of frame - 8 bits
  public static final char FLAGS           = (char) (0x00 >> 2) & 0xFC;
  // SFrame flags - 6 bits
  public static final char LINK_ID_CONTROL = (char) 0xFF;
  // Link ID 0x00 - 0x08 or 0xFF (control) - 8 bits

  private char             nLink           = 0xFF;
  private char             link            = 0xFF;

  private int              pos             = 0;
  // Start of frame - 8 bits

  private LinkMatik        lm              = null;

  private char[]           buffer          = new char[MAX_BUFFER_SIZE * 5];
  private int              bufferHeadPos   = 0;
  private int              bufferTailPos   = 0;

  public Channel(char link, LinkMatik lm) {
    this.link = link;
    this.nLink ^= this.link;
    this.lm = lm;
  }

  private void builtFrame(char[] command, int offset) {
    pos = 0;
    frame[pos++] = SOF;
    frame[pos++] = link;
    frame[pos++] = FLAGS;
    frame[pos++] = (char) ((offset) & 0xFF);
    while (pos < offset + CONTROL_OFFSET) {
      frame[pos] = (char) command[pos++ - CONTROL_OFFSET];
    }
    frame[pos] = nLink;
  }

  //char[] returnCharArray = new char[MAX_BUFFER_SIZE];
  char   tempChar;
  char   i;
  public int getNextFrame(char[] frame) {
    i = 0;
    tempChar = buffer[bufferHeadPos];
    if(bufferHeadPos == bufferTailPos){
      //buffer emtpy
      return 0;
    }
    if ((tempChar + bufferHeadPos) > bufferTailPos) {
      // error
      bufferHeadPos = bufferTailPos;
      return 0;
    }
    if(frame.length < tempChar){
      frame = new char[MAX_BUFFER_SIZE];
    }
    while (i <= tempChar) {
      frame[i++] = buffer[bufferHeadPos++];
    }
    tempChar ^= buffer[bufferHeadPos++];
    if (tempChar != 0xff) {
      // ERROR
      bufferHeadPos = bufferTailPos;
      return 0;
    }
    return --i;
  }

  public void addToBuffer(char[] payload) {
    // {7,4,2,9}
    buffer[bufferTailPos++] = (char) payload.length;
    for (char i = 0; i < payload.length; i++) {
      buffer[bufferTailPos++] = payload[i];
    }
    buffer[bufferTailPos++] = (char) (0xff ^ payload.length);
  }

  public int receivebytes(char[] bytes) {
    // TODO Auto-generated method stub
    lm.receiveFrame();
    return getNextFrame(frame);
  }

  public int receivebytes(char[] bytes, int timeout) {
    // TODO Auto-generated method stub
    lm.receiveFrame(timeout);
    return getNextFrame(frame);

  }

  public boolean sendbytes(char[] bytes, int offset) {
    if (offset > MAX_DATA_SIZE) { return false; // EXIT
    }
    builtFrame(bytes, offset);
    lm.send(frame, pos + 1);
    return true;
  }

}